@@ -8,6 +8,7 @@ from .checks.base import QueryCheck
 from .checks.base import RangeCheck
 from .checks.base import UniqueCheck
 from .checks.cross_section_definitions import CrossSectionEqualElementsCheck
+from .checks.cross_section_definitions import CrossSectionExpectEmptyCheck
 from .checks.cross_section_definitions import (
     CrossSectionFirstElementNonZeroCheck,
 )
@@ -17,6 +18,13 @@ from .checks.cross_section_definitions import CrossSectionFloatListCheck
 from .checks.cross_section_definitions import CrossSectionGreaterZeroCheck
 from .checks.cross_section_definitions import CrossSectionIncreasingCheck
 from .checks.cross_section_definitions import CrossSectionNullCheck
+from .checks.cross_section_definitions import (
+    CrossSectionYZCoordinateCountCheck,
+)
+from .checks.cross_section_definitions import CrossSectionYZHeightCheck
+from .checks.cross_section_definitions import (
+    CrossSectionYZIncreasingWidthIfOpenCheck,
+)
 from .checks.factories import generate_enum_checks
 from .checks.factories import generate_foreign_key_checks
 from .checks.factories import generate_geometry_checks
@@ -30,6 +38,8 @@ from .checks.other import ConnectionNodesLength
 from .checks.other import CrossSectionLocationCheck
 from .checks.other import LinestringLocationCheck
 from .checks.other import OpenChannelsWithNestedNewton
+from .checks.other import PotentialBreachInterdistanceCheck
+from .checks.other import PotentialBreachStartEndCheck
 from .checks.other import SpatialIndexCheck
 from .checks.other import Use0DFlowCheck
 from .checks.raster import GDALAvailableCheck
@@ -51,6 +61,9 @@ from sqlalchemy.orm import Query
 from typing import List
 
 
+TOLERANCE_M = 1.0
+
+
 def is_none_or_empty(col):
     return (col == None) | (col == "")
 
@@ -256,7 +269,7 @@ CHECKS += [
 
 CHECKS += [
     CrossSectionLocationCheck(
-        level=CheckLevel.WARNING, max_distance=1.0, error_code=52
+        level=CheckLevel.WARNING, max_distance=TOLERANCE_M, error_code=52
     ),
     OpenChannelsWithNestedNewton(error_code=53),
     QueryCheck(
@@ -349,6 +362,7 @@ CHECKS += [
             constants.CrossSectionShape.CLOSED_RECTANGLE,
             constants.CrossSectionShape.TABULATED_RECTANGLE,
             constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+            constants.CrossSectionShape.TABULATED_YZ,
         ),
     ),
     CrossSectionFloatCheck(
@@ -374,6 +388,7 @@ CHECKS += [
             constants.CrossSectionShape.CIRCLE,
             constants.CrossSectionShape.CLOSED_RECTANGLE,
             constants.CrossSectionShape.EGG,
+            constants.CrossSectionShape.INVERTED_EGG,
         ),
     ),
     CrossSectionGreaterZeroCheck(
@@ -387,6 +402,7 @@ CHECKS += [
         shapes=(
             constants.CrossSectionShape.TABULATED_RECTANGLE,
             constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+            constants.CrossSectionShape.TABULATED_YZ,
         ),
     ),
     CrossSectionFloatListCheck(
@@ -395,6 +411,7 @@ CHECKS += [
         shapes=(
             constants.CrossSectionShape.TABULATED_RECTANGLE,
             constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+            constants.CrossSectionShape.TABULATED_YZ,
         ),
     ),
     CrossSectionEqualElementsCheck(
@@ -402,6 +419,7 @@ CHECKS += [
         shapes=(
             constants.CrossSectionShape.TABULATED_RECTANGLE,
             constants.CrossSectionShape.TABULATED_TRAPEZIUM,
+            constants.CrossSectionShape.TABULATED_YZ,
         ),
     ),
     CrossSectionIncreasingCheck(
@@ -426,6 +444,29 @@ CHECKS += [
             constants.CrossSectionShape.TABULATED_TRAPEZIUM,
         ),
     ),
+    CrossSectionExpectEmptyCheck(
+        error_code=94,
+        level=CheckLevel.WARNING,
+        column=models.CrossSectionDefinition.width,
+        shapes=(
+            constants.CrossSectionShape.CIRCLE,
+            constants.CrossSectionShape.EGG,
+            constants.CrossSectionShape.INVERTED_EGG,
+        ),
+    ),
+    CrossSectionYZHeightCheck(
+        error_code=95,
+        column=models.CrossSectionDefinition.height,
+        shapes=(constants.CrossSectionShape.TABULATED_YZ,),
+    ),
+    CrossSectionYZCoordinateCountCheck(
+        error_code=96,
+        shapes=(constants.CrossSectionShape.TABULATED_YZ,),
+    ),
+    CrossSectionYZIncreasingWidthIfOpenCheck(
+        error_code=97,
+        shapes=(constants.CrossSectionShape.TABULATED_YZ,),
+    ),
 ]
 
 
@@ -692,6 +733,167 @@ CHECKS += [
 ]
 
 
+## 026x: Exchange lines
+CHECKS += [
+    QueryCheck(
+        error_code=260,
+        level=CheckLevel.ERROR,
+        column=models.Channel.id,
+        invalid=Query(models.Channel)
+        .join(models.ExchangeLine)
+        .filter(
+            models.Channel.calculation_type.notin_(
+                {
+                    constants.CalculationType.CONNECTED,
+                    constants.CalculationType.DOUBLE_CONNECTED,
+                }
+            )
+        ),
+        message="v2_channel can only have a v2_exchange_line if it has "
+        "a (double) connected (102 or 105) calculation type",
+    ),
+    QueryCheck(
+        error_code=261,
+        level=CheckLevel.ERROR,
+        column=models.Channel.id,
+        invalid=Query(models.Channel)
+        .join(models.ExchangeLine)
+        .filter(
+            models.Channel.calculation_type == constants.CalculationType.CONNECTED,
+        )
+        .group_by(models.ExchangeLine.channel_id)
+        .having(func.count(models.ExchangeLine.id) > 1),
+        message="v2_channel can have max 1 v2_exchange_line if it has "
+        "connected (102) calculation type",
+    ),
+    QueryCheck(
+        error_code=262,
+        level=CheckLevel.ERROR,
+        column=models.Channel.id,
+        invalid=Query(models.Channel)
+        .join(models.ExchangeLine)
+        .filter(
+            models.Channel.calculation_type
+            == constants.CalculationType.DOUBLE_CONNECTED,
+        )
+        .group_by(models.ExchangeLine.channel_id)
+        .having(func.count(models.ExchangeLine.id) > 2),
+        message="v2_channel can have max 2 v2_exchange_line if it has "
+        "double connected (105) calculation type",
+    ),
+    QueryCheck(
+        error_code=263,
+        level=CheckLevel.WARNING,
+        column=models.ExchangeLine.the_geom,
+        invalid=Query(models.ExchangeLine)
+        .join(models.Channel)
+        .filter(
+            geo_query.length(models.ExchangeLine.the_geom)
+            < (0.8 * geo_query.length(models.Channel.the_geom))
+        ),
+        message=(
+            "v2_exchange_line.the_geom should not be significantly shorter than its "
+            "corresponding channel."
+        ),
+    ),
+    QueryCheck(
+        error_code=264,
+        level=CheckLevel.WARNING,
+        column=models.ExchangeLine.the_geom,
+        invalid=Query(models.ExchangeLine)
+        .join(models.Channel)
+        .filter(
+            geo_query.distance(models.ExchangeLine.the_geom, models.Channel.the_geom)
+            > 500.0
+        ),
+        message=(
+            "v2_exchange_line.the_geom is far (> 500 m) from its corresponding channel"
+        ),
+    ),
+]
+
+## 027x: Potential breaches
+CHECKS += [
+    QueryCheck(
+        error_code=270,
+        level=CheckLevel.ERROR,
+        column=models.PotentialBreach.id,
+        invalid=Query(models.PotentialBreach)
+        .join(models.Channel)
+        .filter(
+            models.Channel.calculation_type.notin_(
+                {
+                    constants.CalculationType.CONNECTED,
+                    constants.CalculationType.DOUBLE_CONNECTED,
+                }
+            )
+        ),
+        message="v2_potential_breach is assigned to an isolated "
+        "or embedded channel.",
+    ),
+    QueryCheck(
+        error_code=271,
+        level=CheckLevel.ERROR,
+        column=models.PotentialBreach.id,
+        invalid=Query(models.PotentialBreach)
+        .join(models.Channel)
+        .filter(
+            models.Channel.calculation_type == constants.CalculationType.CONNECTED,
+        )
+        .group_by(
+            models.PotentialBreach.channel_id,
+            func.PointN(models.PotentialBreach.the_geom, 1),
+        )
+        .having(func.count(models.PotentialBreach.id) > 1),
+        message="v2_channel can have max 1 v2_potential_breach at the same position "
+        "on a channel of connected (102) calculation type",
+    ),
+    QueryCheck(
+        error_code=272,
+        level=CheckLevel.ERROR,
+        column=models.PotentialBreach.id,
+        invalid=Query(models.PotentialBreach)
+        .join(models.Channel)
+        .filter(
+            models.Channel.calculation_type
+            == constants.CalculationType.DOUBLE_CONNECTED,
+        )
+        .group_by(
+            models.PotentialBreach.channel_id,
+            func.PointN(models.PotentialBreach.the_geom, 1),
+        )
+        .having(func.count(models.PotentialBreach.id) > 2),
+        message="v2_channel can have max 2 v2_potential_breach at the same position "
+        "on a channel of double connected (105) calculation type",
+    ),
+    QueryCheck(
+        error_code=273,
+        level=CheckLevel.ERROR,
+        column=models.PotentialBreach.id,
+        invalid=Query(models.PotentialBreach)
+        .join(models.Channel)
+        .filter(
+            geo_query.distance(
+                func.PointN(models.PotentialBreach.the_geom, 1), models.Channel.the_geom
+            )
+            > TOLERANCE_M
+        ),
+        message="v2_potential_breach.the_geom must begin at the channel it is assigned to",
+    ),
+    PotentialBreachStartEndCheck(
+        error_code=274,
+        level=CheckLevel.ERROR,
+        column=models.PotentialBreach.the_geom,
+        min_distance=TOLERANCE_M,
+    ),
+    PotentialBreachInterdistanceCheck(
+        error_code=275,
+        level=CheckLevel.ERROR,
+        column=models.PotentialBreach.the_geom,
+        min_distance=TOLERANCE_M,
+    ),
+]
+
 ## 030x: SETTINGS
 
 CHECKS += [
